package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import opengl.RectangleRenderer;
import opengl.TextRenderer;

import logic.code.CodeItem;

public class ColorCheckbox extends JPanel implements ActionListener {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 2386101848611842800L;
	private Color color = Color.lightGray;
	private JCheckBox checkBox;
	private JLabel label;
	private ColorQuad colorQuad;
	private int statementType = CodeItem.UNDEFINED;
	
	public ColorCheckbox(String label, int statementType){
		setLayout(new FlowLayout(FlowLayout.LEFT, 0, -1));
		
		addCheckbox();
		addQuad();
		addLabel(label);
		this.statementType = statementType;
		
	}
	
	public ColorCheckbox(String label, int statementType, Color color){
		setLayout(new FlowLayout(FlowLayout.LEFT, 0, -1));
		
		addCheckbox();
		addQuad();
		addLabel(label);
		this.statementType = statementType;
		colorChanged(color);
		
	}
	
	private void addCheckbox(){
		checkBox = new JCheckBox();
		checkBox.setSelected(true);
		checkBox.addActionListener(this);
		add(checkBox);
	}
	
	private void addQuad(){
		// add a dummy Panel (for alignment):
		JPanel dummy = new JPanel();
		dummy.setSize(new Dimension(6, 6));
		
		colorQuad = new ColorQuad(this);
		dummy.add(colorQuad);
		add(dummy);
	}
	
	private void addLabel(String labelStr){
		label = new JLabel(labelStr);
		add(label);
	}
	
	public Color getColor(){
		return color;
	}
	
	public void colorChanged(Color color){
		this.color = color;
		colorQuad.updateColor();
		
		if (checkBox.getActionCommand().equals("Text"))
			TextRenderer.setTextColor(color);
		else
			RectangleRenderer.colorsChanged(true);
	}
	
	public JCheckBox getCheckbox(){
		return checkBox;
	}
	
	public void setAcitonCommand(String actionCommand) {
		checkBox.setActionCommand(actionCommand);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getActionCommand().equals("Text"))
			TextRenderer.animateText(checkBox.isSelected());
		else
			RectangleRenderer.animateStatement(statementType, checkBox.isSelected());
		
	}
	

}
	
class ColorQuad extends JPanel implements MouseListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5499689016834150587L;
	private static int length = 5;
	
	private ColorCheckbox parent;

	public ColorQuad(ColorCheckbox parent){
		this.parent = parent;
		addMouseListener(this);
		setBackground(parent.getColor());
		setSize(new Dimension(length, length));
		setBorder(BorderFactory.createLoweredBevelBorder());
	}
	
	public void updateColor(){
		setBackground(parent.getColor());
	}
	
// MOUSELISTENER-METHODS:
	
	public void mouseClicked(MouseEvent me) { 
		parent.getCheckbox().requestFocus();
		if (me.getClickCount() == 2){
			Color newColor = JColorChooser.showDialog(new JFrame(), 
		              "Select color...", parent.getColor());
			if (newColor != null){
				parent.colorChanged(newColor);
			}
				
		}
	}
	
	public void mousePressed(MouseEvent me)  {
	
	}

	public void mouseEntered(MouseEvent me) {
		
	}

	public void mouseExited(MouseEvent me) {
		
	}
	
	public void mouseReleased(MouseEvent e) {
		
	}
	
}
