package gui;

import gui.common.FileInfo;
import gui.common.PackageInfo;
import gui.common.SyntaxEditor;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Writer;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import opengl.CodeRenderer;

import logic.code.CodeItem;
import logic.code.CodeItemFinder;

public class EditorPanel extends JPanel implements ActionListener, ChangeListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -2393041852675976464L;
	private static EditorPanel panel;
	private static JTabbedPane tabbedPane;
	private static PackageInfo activePackageInfo = null;
	private static JLabel lineLabel;
	private static Button showItemButton, hideItemButton;
	private static FileInfo singleFile;
	private static FileInfo lastSelectedFileInfo = null;
	
	private EditorPanel(){
		super(new BorderLayout());
		JPanel topPanel = new JPanel(new GridLayout(1, 1));
		
		tabbedPane = new JTabbedPane();
		tabbedPane.addChangeListener(this);
		topPanel.add(tabbedPane);

		JPanel buttomPanel = new JPanel(new FlowLayout(FlowLayout.LEFT)); 

		showItemButton = new Button("Show Statement");
		showItemButton.addActionListener(this);
		showItemButton.setActionCommand("ShowStatement");
		showItemButton.setEnabled(false);
		buttomPanel.add(showItemButton);
		
		hideItemButton = new Button("Hide Statement");
		hideItemButton.addActionListener(this);
		hideItemButton.setActionCommand("HideStatement");		
		hideItemButton.setEnabled(false);
		buttomPanel.add(hideItemButton);
		
		lineLabel = new JLabel();
		buttomPanel.add(lineLabel);
		
		add(topPanel, BorderLayout.CENTER);
		add(buttomPanel, BorderLayout.SOUTH);
	}
	
	public static void setSingleFile(FileInfo fileInfo) {
		singleFile = fileInfo;
		flushTabbedPane();
		setFile(singleFile.getEditor());
	}
	
	private static void flushTabbedPane() {
		tabbedPane.removeAll();
		showItemButtonEnabled(false);
	}
	
	public static EditorPanel getEditorPanel() {
		
		if (panel == null)
			panel = new EditorPanel();
		
		return panel;
	}
	
	public static void lineNumberChanged(int lineNumber){
		lineLabel.setText("Line: " + lineNumber);
		
		
	}
	
	public static void setActivePackage(PackageInfo pInfo){
		
		flushTabbedPane();
		
		for (int i = 0; i < pInfo.numFiles(); i++){
			tabbedPane.addTab(pInfo.getFiles().get(i).getEditor().getTitle(), pInfo.getFiles().get(i).getEditor());
		}
		
		activePackageInfo = pInfo;
	}
	
	public static void titleChange(SyntaxEditor editor){
		tabbedPane.setTitleAt(tabbedPane.indexOfComponent(editor), editor.getTitle());
	}
	
	public static void setFile(SyntaxEditor editor) {
		tabbedPane.addTab(editor.getTitle(), editor);
	}
	
	public static void getNearestItem(){
		
	}

	public static void showItemButtonEnabled(boolean enabled) {
		showItemButton.setEnabled(enabled);
	}
	
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		
		
		if (ae.getActionCommand().equals("ShowStatement")){
			
			int index = tabbedPane.getSelectedIndex();
			FileInfo fileInfo = singleFile;
			
			if (activePackageInfo != null) {
				activePackageInfo.setActiveFileInfoIndex(index);
				fileInfo = activePackageInfo.getActiveFileInfo();
			}
				
			int lineNumber = fileInfo.getEditor().getCaretLinePosition();
			
			

			CodeItem item = fileInfo.getCodeItem();
				
			item = CodeItemFinder.getItem(item, lineNumber);
			
			if (!hideItemButton.isEnabled())
				CodeRenderer.setSelectEffect(true);
			
			CodeRenderer.setSelectedItem(item, tabbedPane.getSelectedIndex());
			fileInfo.getEditor().getTextPane().setSelection(item.firstLine(), item.lastLine());
			hideItemButton.setEnabled(true);
			
			lastSelectedFileInfo = fileInfo;
			
		}
		else if (ae.getActionCommand().equals("HideStatement")){
			
			CodeRenderer.setSelectEffect(false);
			hideItemButton.setEnabled(false);
			if (lastSelectedFileInfo != null){
				lastSelectedFileInfo.getEditor().getTextPane().setSelected(false);
				lastSelectedFileInfo.getEditor().getTextPane().updateUI();
			}
		}
		
	}

	@Override
	public void stateChanged(ChangeEvent ce) {

		if (tabbedPane.getSelectedIndex() == -1)
			return;
		
		if (activePackageInfo != null)
			activePackageInfo.setActiveFileInfoIndex(tabbedPane.getSelectedIndex());
		
		if (hideItemButton.isEnabled()) {
			CodeRenderer.setSelectEffect(false);
			hideItemButton.setEnabled(false);
			if (lastSelectedFileInfo != null){
				lastSelectedFileInfo.getEditor().getTextPane().setSelected(false);
				lastSelectedFileInfo.getEditor().getTextPane().updateUI();
			}
			
		}
		
		if (activePackageInfo != null) 
			PackagePanel.updateTreeSelection();
	}
	
	public static void activateTab(int index){
		
		if (index == -1)
			return;
		
		if (tabbedPane.getSelectedIndex() == index)
			return;
		
		tabbedPane.setSelectedIndex(index);
		
	}
	
	public static void saveActiveFile(){
		
		int index = tabbedPane.getSelectedIndex();
		FileInfo fileInfo;
		Boolean saved = true;
		
		if (activePackageInfo != null) {
			activePackageInfo.setActiveFileInfoIndex(index);
			fileInfo = activePackageInfo.getActiveFileInfo();
		}
		else
			fileInfo = singleFile;
		
		if (fileInfo != null){
		
			PrintWriter pw = null;

			try{
				
				Writer fw = new FileWriter(fileInfo.getAbsoluteFileName()); 
				Writer bw = new BufferedWriter( fw ); 
				pw = new PrintWriter(bw);

				int searchPos = 0;
				String text = fileInfo.getEditor().getTextPane().getText();
				
				while (true){
					int searchEnd = text.indexOf("\n", searchPos);
					if (searchEnd != -1){
						pw.println(text.substring(searchPos, searchEnd));
					
						System.out.println(text.substring(searchPos, searchEnd));
					}
					else
						break;
					
					searchPos = searchEnd + 1;
				}
			}
			catch (Exception e) { 
				
				saved = false;
				
				String errorMessage = "I could not wirte to the file: " + fileInfo.getName();
				JOptionPane.showMessageDialog(GuiMain.getMainFrame(), errorMessage, "Error", JOptionPane.OK_OPTION);
				
				e.printStackTrace();
			} 

			finally { 
				if (pw != null) 
					pw.close();
				
				
			}
			if (saved){
				fileInfo.getEditor().save();
				fileInfo.refreshItem();
				if (activePackageInfo != null){ 
					CodeRenderer.setPackage(activePackageInfo);
				}
				else{
					CodeRenderer.setSingleFile(fileInfo);
				}
					
			}

		}
	}
	
}

