package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.UIManager;

import logic.CodeVisMain;

public class GuiMain extends JFrame{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 8144814164235649038L;

	public static final String title = "Code Vis";
	
	private static GuiMain frame = null;
	private static GLCanvas canvas = null;
	
	private GuiMain() {

		super(title);

		try{
			// define os look and feel
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e){
			// do nothing, java loads the default look and feel
			System.out.println("I didn't load the Default System LookAndFeel...");
		}
		
		Toolkit.getDefaultToolkit().setDynamicLayout(true);				
					
		this.setSize(new Dimension(1210, 710));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLayout(new BorderLayout(0,0));
		
		// add panels...

		JPopupMenu.setDefaultLightWeightPopupEnabled(false);  // menus will appear above the canvas 
		this.setJMenuBar(MenuBar.getMenu());
		
		getCanvas().setMinimumSize(new Dimension(0, 0));
		JPanel canvasPanel = new JPanel();
		canvasPanel.setLayout(new BorderLayout());
		canvasPanel.add(getCanvas());
		

		JSplitPane horizontalSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				canvasPanel, EditorPanel.getEditorPanel());
		horizontalSplitPane.setOneTouchExpandable(true);
		horizontalSplitPane.setDividerSize(8);
		horizontalSplitPane.setResizeWeight(1);


		JPanel dummy = new JPanel(new BorderLayout());
		dummy.setPreferredSize(new Dimension(200, 0));

		dummy.add(PackagePanel.getPackageExplorerPanel(), BorderLayout.CENTER);
		dummy.add(StatementPanel.getStatementPanel(), BorderLayout.SOUTH);
		
		
		JSplitPane verticalSplitPane  = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				dummy, horizontalSplitPane);
		
		verticalSplitPane.setOneTouchExpandable(true);
		verticalSplitPane.setDividerSize(8);
		
		this.add(verticalSplitPane, BorderLayout.CENTER);

		this.setVisible(true);
		
		System.out.println("Main Frame initialized...");
	}
	
	public static GLCanvas getCanvas() {

		if (canvas == null) {
			GLCapabilities caps = new GLCapabilities();
			caps.setDoubleBuffered(true);
			caps.setHardwareAccelerated(true);
			canvas = new GLCanvas(caps);
						
			canvas.setPreferredSize(new Dimension(800,800));
			canvas.addGLEventListener(CodeVisMain.getCanvas());
			canvas.addMouseListener(MainFrameListener.getMainFrameListener());
			canvas.addMouseMotionListener(MainFrameListener.getMainFrameListener());
			canvas.addMouseWheelListener(MainFrameListener.getMainFrameListener());
			canvas.addKeyListener(MainFrameListener.getMainFrameListener());
		}
		
		return canvas;
	}
	
	
	/**
	 * if no instance of MainFrame exists, it creates a new instance;
	 * else it returns the existing one
	 * 
	 * @return the one and only instance of the MainFrame
	 */
	public static synchronized GuiMain getMainFrame() {
		
		if(frame == null) {
			frame = new GuiMain();
		}
		
		return frame;
	}
}
