package gui;

import gui.filters.CodeFileChooser;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import opengl.CodeRenderer;
import opengl.DetailView;
import opengl.VisCanvas;

public class MainFrameListener implements ActionListener, MouseListener, MouseMotionListener, MouseWheelListener, KeyListener{

	private static MainFrameListener listener = null;
	
	private MainFrameListener() {
		
	}
	
	public static synchronized MainFrameListener getMainFrameListener() {
		
		if (listener == null)
			listener = new MainFrameListener();
		
		return listener;
	}
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		
		// insert actions here
		if(ae.getActionCommand().equals("MenuOpenFile")){
			CodeFileChooser.getFileChooser().readTextFile();
        }
		else if(ae.getActionCommand().equals("MenuSaveStatementLook")){
			CodeFileChooser.getFileChooser().writeStatementLook();
        }
		else if(ae.getActionCommand().equals("MenuOpenStatementLook")){
			CodeFileChooser.getFileChooser().readStatementLook();
        }
		else if(ae.getActionCommand().equals("ImportProject")){
			CodeFileChooser.getFileChooser().importProject();
        }
		else if(ae.getActionCommand().equals("MenuResetCanvas")) {
			VisCanvas.flushCamera();
		}
		else if(ae.getActionCommand().equals("MenuZoomIn")) {
			VisCanvas.zoomObject(-1);
		}
		else if(ae.getActionCommand().equals("MenuZoomOut")) {
			VisCanvas.zoomObject(1);
		}
		else if(ae.getActionCommand().equals("MenuSaveFile")){
			EditorPanel.saveActiveFile();
		}
		else if(ae.getActionCommand().equals("MenuShowFile")){
			CodeRenderer.showFile();
		}
		else if(ae.getActionCommand().equals("ScreenShot")){
			CodeFileChooser.getFileChooser().saveScreenShot();
		}
	}

	@Override
	public void mouseDragged(MouseEvent me) {
		
		if(me.isControlDown())
			DetailView.translateCamera(me.getX(), me.getY());	
		else if(me.isShiftDown())
			VisCanvas.translateObject(me.getX(), me.getY());
		
	}

	@Override
	public void mouseMoved(MouseEvent me) {
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent mwe) {
		VisCanvas.zoomObject(mwe.getWheelRotation());		
	}

	@Override
	public void keyPressed(KeyEvent ke) {
		if(ke.getKeyCode() == KeyEvent.VK_UP)
			VisCanvas.zoomObject(1);
		else if(ke.getKeyCode() == KeyEvent.VK_DOWN)
			VisCanvas.zoomObject(-1);
	}

	@Override
	public void keyReleased(KeyEvent ke) {
		if(ke.getKeyCode() == KeyEvent.VK_CONTROL) {
			DetailView.renderMode(false);
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}

	
	// Mouse Listener
	@Override
	public void mouseClicked(MouseEvent me) {
	}

	@Override
	public void mouseEntered(MouseEvent me) {
	}

	@Override
	public void mouseExited(MouseEvent me) {
		DetailView.renderMode(false);
	}

	@Override
	public void mousePressed(MouseEvent me) {
		if (me.isControlDown() && me.getButton() != MouseEvent.BUTTON3)
			DetailView.renderMode(true);
		
		if (me.getButton() == MouseEvent.BUTTON2)
			DetailView.drawRects(false);
		else
			DetailView.drawRects(true);
	}

	@Override
	public void mouseReleased(MouseEvent me) {
		if (me.isControlDown())
			DetailView.renderMode(false);
		
	}
	
}
