package gui;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

public class MenuBar extends JMenuBar{
	
	public static final int OPEN_ITEM = 0, SAVE_ITEM = 1;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 6293763939885061751L;
	
	private static MenuBar menu = null;
	
	// File menu items
	private JMenuItem loadTextFileItem = null;
	private JMenuItem saveTextFileItem = null;
	private JMenuItem loadProject = null;
	private JMenuItem loadStatementLookItem = null;
	private JMenuItem saveStatementLookItem = null;
	
	
	/**
	 * Creates the Menu
	 * 
	 * @return a JMenuBar holding all assigned menu items
	 */
	private MenuBar() {
		
		loadTextFileItem = new JMenuItem("Open File");
		loadTextFileItem.setMnemonic(KeyEvent.VK_O);
		loadTextFileItem.setActionCommand("MenuOpenFile");
		loadTextFileItem.setAccelerator(KeyStroke.getKeyStroke('O', InputEvent.CTRL_DOWN_MASK));
		loadTextFileItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		saveTextFileItem = new JMenuItem("Save File");
		saveTextFileItem.setMnemonic(KeyEvent.VK_S);
		saveTextFileItem.setActionCommand("MenuSaveFile");
		saveTextFileItem.setAccelerator(KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK));
		saveTextFileItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		JMenuItem saveScreenShotItem = new JMenuItem("Save Screenshot");
		saveScreenShotItem.setMnemonic(KeyEvent.VK_C);
		saveScreenShotItem.setActionCommand("ScreenShot");
		saveScreenShotItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, InputEvent.CTRL_DOWN_MASK));
		saveScreenShotItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		loadProject = new JMenuItem("Import Project");
		loadProject.setMnemonic(KeyEvent.VK_I);
		loadProject.setActionCommand("ImportProject");
		loadProject.setAccelerator(KeyStroke.getKeyStroke('I', InputEvent.CTRL_DOWN_MASK));
		loadProject.addActionListener(MainFrameListener.getMainFrameListener());

		loadStatementLookItem = new JMenuItem("Open Statement Look");
		loadStatementLookItem.setMnemonic(KeyEvent.VK_P);
		loadStatementLookItem.setActionCommand("MenuOpenStatementLook");
		loadStatementLookItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		saveStatementLookItem = new JMenuItem("Save Statement Look");
		saveStatementLookItem.setMnemonic(KeyEvent.VK_A);
		saveStatementLookItem.setActionCommand("MenuSaveStatementLook");
		saveStatementLookItem.addActionListener(MainFrameListener.getMainFrameListener());		

		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);

		fileMenu.add(loadProject);
		fileMenu.add(loadTextFileItem);
		fileMenu.add(saveTextFileItem);
		fileMenu.add(saveScreenShotItem);
		fileMenu.add(loadStatementLookItem);
		fileMenu.add(saveStatementLookItem);
		
		// MENU FILE -------------------------------------------------------------------------------
		JMenuItem resetCanvasItem = new JMenuItem("Reset Canvas");
		resetCanvasItem.setMnemonic(KeyEvent.VK_P);
		resetCanvasItem.setActionCommand("MenuResetCanvas");
		resetCanvasItem.setAccelerator(KeyStroke.getKeyStroke('0', InputEvent.CTRL_DOWN_MASK));
		resetCanvasItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		JMenuItem zoomInItem = new JMenuItem("Zoom in");
		zoomInItem.setMnemonic(KeyEvent.VK_I);
		zoomInItem.setActionCommand("MenuZoomIn");
		zoomInItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, InputEvent.CTRL_DOWN_MASK));
		zoomInItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		JMenuItem zoomOutItem = new JMenuItem("Zoom out");
		zoomOutItem.setMnemonic(KeyEvent.VK_O);
		zoomOutItem.setActionCommand("MenuZoomOut");
		zoomOutItem.setAccelerator(KeyStroke.getKeyStroke('-', InputEvent.CTRL_DOWN_MASK));
		zoomOutItem.addActionListener(MainFrameListener.getMainFrameListener());
		
		JMenuItem showFileItem = new JMenuItem("Show File");
		showFileItem.setMnemonic(KeyEvent.VK_F);
		showFileItem.setActionCommand("MenuShowFile");
		showFileItem.setAccelerator(KeyStroke.getKeyStroke('1', InputEvent.CTRL_DOWN_MASK));
		showFileItem.addActionListener(MainFrameListener.getMainFrameListener());

		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		
		viewMenu.add(resetCanvasItem);
		viewMenu.add(showFileItem);
		viewMenu.add(zoomInItem);
		viewMenu.add(zoomOutItem);
		
		
		this.add(fileMenu);		
		this.add(viewMenu);
		
	}
	
	public static synchronized MenuBar getMenu() {
		
		if (menu == null)
			menu = new MenuBar();
		
		return menu;
	}
	
	public JMenuItem getMenuItem(int which) {
		
		switch (which) {
		case OPEN_ITEM:
			return loadTextFileItem;
		case SAVE_ITEM:
			return saveTextFileItem;
		}
		
		return null;
	}
}
