package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.nio.FloatBuffer;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import opengl.RectangleRenderer;
import opengl.TextRenderer;
import opengl.VisCanvas;

import logic.code.CodeItem;

public class StatementPanel extends JPanel implements ActionListener, ChangeListener {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -1359524600716876371L;
	private static StatementPanel panel = null;
	private static LinkedList<ColorCheckbox> boxes = null;
	
	private StatementPanel(){
		
		// better 2 columns -> package explorer needs horizontal space anyhow
		
		setLayout(new BorderLayout());

		add(getAppearancePanel(), BorderLayout.NORTH);
		
		JPanel statementsPanel = new JPanel (new GridLayout(6, 2));
		statementsPanel.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Statements"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		
		// add here all the statement-types defined in CodeItem
		boxes = new LinkedList<ColorCheckbox>();
		boxes.add(new ColorCheckbox("background",	CodeItem.UNDEFINED,		new Color(200, 200, 200)));
		boxes.add(new ColorCheckbox("package", 		CodeItem.PACKAGE, 		new Color(17, 17, 200)));
		boxes.add(new ColorCheckbox("import",		CodeItem.IMPORT,		new Color(52, 52, 130)));
		boxes.add(new ColorCheckbox("comment",		CodeItem.COMMENT,		new Color(87, 180, 100)));
		boxes.add(new ColorCheckbox("class",		CodeItem.CLASS,			new Color(127, 147, 150)));
		boxes.add(new ColorCheckbox("method",		CodeItem.METHOD,		new Color(120, 0, 0)));
		boxes.add(new ColorCheckbox("loop",			CodeItem.LOOP,			new Color(0, 0, 100)));
		boxes.add(new ColorCheckbox("switch",		CodeItem.SWITCH,		new Color(0, 100, 92)));
		boxes.add(new ColorCheckbox("if",			CodeItem.IF,			new Color(255, 150, 0)));
		boxes.add(new ColorCheckbox("try",			CodeItem.TRY,			new Color(255, 0, 0)));
		boxes.add(new ColorCheckbox("GL",			CodeItem.GL_MODE,		new Color(100, 100, 100)));
		
		Iterator<ColorCheckbox> boxesIter = boxes.iterator();
		
		// append boxes to this panel
		while(boxesIter.hasNext()) {
			statementsPanel.add(boxesIter.next());
		}
		
		add(statementsPanel, BorderLayout.SOUTH);
	}
	
	private JPanel getAppearancePanel() {
		
		JPanel globalPanel = new JPanel (new BorderLayout());
		globalPanel.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Appearance"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		
		JPanel textPanel = new JPanel(new BorderLayout(20, 0));
		// TEXT editing... should include transparency
		ColorCheckbox textbox = new ColorCheckbox("Text", CodeItem.UNDEFINED, new Color(255, 255, 255));
		textbox.setAcitonCommand("Text");
		textPanel.add(textbox, BorderLayout.WEST);
		
		JSlider textSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		textSlider.setName("textSlider");
		textSlider.setMajorTickSpacing(50);
		textSlider.setMinorTickSpacing(10);
		textSlider.setPaintTicks(true);
		textSlider.addChangeListener(this);
		
		textPanel.add(textSlider, BorderLayout.CENTER);
		
		// rectangle panel
		JPanel rectPanel = new JPanel(new BorderLayout(9, 0));
		
		JCheckBox rectBox = new JCheckBox("Rectangles");
		rectBox.setSelected(true);
		rectBox.addActionListener(this);
		rectBox.setActionCommand("showRectangles");
		rectPanel.add(rectBox, BorderLayout.WEST);

		JSlider rectSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		rectSlider.setName("rectangleSlider");
		rectSlider.setMajorTickSpacing(50);
		rectSlider.setMinorTickSpacing(10);
		rectSlider.setPaintTicks(true);
		rectSlider.addChangeListener(this);

		rectPanel.add(rectSlider, BorderLayout.CENTER);
		
		globalPanel.add(textPanel, BorderLayout.NORTH);
		globalPanel.add(rectPanel, BorderLayout.SOUTH);
		
		return globalPanel;
	}

	public static LinkedList<ColorCheckbox> getColorCheckBoxes(){
		return boxes;
	}
	
	public static Color[] getColorMapping() {
		
		Iterator<ColorCheckbox> boxesIter = boxes.iterator();
		Color[] colorMap = new Color[256];
		
		for (int i = 0; i < colorMap.length; i++) {
			if (boxesIter.hasNext()){
				Color temp = boxesIter.next().getColor();
				Color color = new Color(temp.getRed()/255f, temp.getGreen()/255f, temp.getBlue()/255f, 1f);
				colorMap[i] = color;
			}
			else
				colorMap[i] = new Color(255,0,0);
		}
		
		return colorMap;
	}
	
	
	public static FloatBuffer getTransferFunction() {
		
		Color[] cHist = getColorMapping();
		float[] transfer = new float[cHist.length*4];
		
		for(int i = 0; i < cHist.length; i++) {
			
			transfer[i*4] = (float)cHist[i].getRed()/255.0f;
			transfer[i*4+1] = (float)cHist[i].getGreen()/255.0f;
			transfer[i*4+2] = (float)cHist[i].getBlue()/255.0f;
			transfer[i*4+3] = (float)cHist[i].getAlpha()/255.0f;
			
		}
				
		return FloatBuffer.wrap(transfer);
	}
	
	public static StatementPanel getStatementPanel(){
		if (panel == null)
			panel = new StatementPanel();
		
		return panel;
	}
	
	public static void saveStatementLook(String fileName){
		
		fileName += ".mft";
		PrintWriter pw = null;

		try{
			Writer fw = new FileWriter(fileName); 
			Writer bw = new BufferedWriter( fw ); 
			pw = new PrintWriter(bw);
			ColorCheckbox checkBox = null;
			for (int i = 0; i < panel.getComponentCount(); i++){
				Component comp = panel.getComponent(i);
				if (comp instanceof ColorCheckbox){
					checkBox = (ColorCheckbox)comp;
					pw.println(checkBox.getCheckbox().isSelected() + "/" + 
							checkBox.getColor().getRed() + "r" + 
							checkBox.getColor().getGreen() + "g" +							
							checkBox.getColor().getBlue() + "b");
				}
			}
		} 
		
		catch (Exception e) {

			String errorMessage = "I could not write to the file: " + fileName;
			JOptionPane.showMessageDialog(GuiMain.getMainFrame(), errorMessage, "Error", JOptionPane.OK_OPTION);
		} 

		finally { 
			if (pw != null) 
				pw.close(); 
		}
	}
	
	public static void openStatementLook(String fileName){
		BufferedReader in = null; 
		

		try{ 
			in = new BufferedReader(new FileReader(fileName)); 

			int pos, posR, posG, posB, i = 0;
			
			ColorCheckbox checkBox;
			
			for (String line; (line = in.readLine()) != null;){
				
				checkBox = (ColorCheckbox) panel.getComponent(i);
				
        		pos = line.indexOf("/");
        		posR = line.indexOf("r", pos);
        		posG = line.indexOf("g", pos);
        		posB = line.indexOf("b", pos);
        		
        		checkBox.getCheckbox().setSelected(Boolean.parseBoolean(line.substring(0, pos)));
        		checkBox.colorChanged(new Color(Integer.parseInt(line.substring(pos+1, posR)),
        				Integer.parseInt(line.substring(posR+1, posG)),
        				Integer.parseInt(line.substring(posG+1, posB))));
        				
        		i++;
	        }
			
		}
	        
	    catch (Exception e){ 
	    	
	    	String errorMessage = "Error while reading file: " + fileName;
	    	JOptionPane.showMessageDialog(GuiMain.getMainFrame(), errorMessage, "Error", JOptionPane.OK_OPTION);
	    } 
	    finally {
	    	if (in != null) {
	    		
	    		try{
	    			in.close();
	    		}
	    		catch(Exception e){
	    		}
	    	}
	     } 
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getActionCommand().equals("showRectangles")) {
			JCheckBox box = (JCheckBox)ae.getSource();
			VisCanvas.drawRectangles(box.isSelected());
		}
	}

	@Override
	public void stateChanged(ChangeEvent ce) {
		JSlider slider = (JSlider) ce.getSource();
		
		if (slider.getName().equals("rectangleSlider"))
			RectangleRenderer.setGlobalAlpha(slider.getValue());
		else if (slider.getName().equals("textSlider"))
			TextRenderer.setGlobalAlpha(slider.getValue());
	}

}
