package gui.common;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;

import java.awt.Color;

import java.util.*;
import java.util.regex.*;

public class SyntaxHighlightingDocument extends DefaultStyledDocument
{
	 /**
	 * 
	 */

	private static final long serialVersionUID = 1L;
	private Element rootElement;
    private HashMap<String,Color> keywords;
    private MutableAttributeSet style;
    
    private Color commentColor = new Color(63, 127, 95);
    private Color multiLineCommentColor = new Color(63, 125, 224);
    private Pattern singleLineCommentDelimter = Pattern.compile("//");
    private Pattern multiLineCommentDelimiterStart = Pattern.compile("/\\*");
    private Pattern multiLineCommentDelimiterEnd = Pattern.compile("\\*/");

    public SyntaxHighlightingDocument() {
    
        putProperty( DefaultEditorKit.EndOfLineStringProperty, "\n" );

        rootElement = getDefaultRootElement();

        Color defaultCol = new Color(163, 0, 116);
        keywords = new HashMap<String, Color>();
        keywords.put( "abstract", defaultCol);
        keywords.put( "interface", defaultCol);
        keywords.put( "class", defaultCol);
        keywords.put( "extends", defaultCol);
        keywords.put( "implements", defaultCol);

        keywords.put( "package", defaultCol);
        keywords.put( "import", defaultCol);
        keywords.put( "private", defaultCol);
        keywords.put( "protected", defaultCol);
        keywords.put( "public", defaultCol);

        keywords.put( "void", defaultCol);
        keywords.put( "boolean", defaultCol);
        keywords.put( "char", defaultCol);
        keywords.put( "byte", defaultCol);
        keywords.put( "float", defaultCol);
        keywords.put( "double", defaultCol);
        keywords.put( "long", defaultCol);
        keywords.put( "short", defaultCol);
        keywords.put( "int", defaultCol);
        
        keywords.put( "true", defaultCol);
        keywords.put( "false", defaultCol);
        keywords.put( "const", defaultCol);
        keywords.put( "null", defaultCol);
        
     

        keywords.put( "break", defaultCol);
        keywords.put( "case", defaultCol);
        keywords.put( "catch", defaultCol);
        keywords.put( "operator", defaultCol);
        keywords.put( "continue", defaultCol);
        keywords.put( "default", defaultCol);
        keywords.put( "do", defaultCol);
        keywords.put( "else", defaultCol);
        keywords.put( "final", defaultCol);
        keywords.put( "finally", defaultCol);
        keywords.put( "for", defaultCol);
        keywords.put( "if", defaultCol);
        keywords.put( "instanceof", defaultCol);
        keywords.put( "native", defaultCol);
        keywords.put( "new", defaultCol);        
        keywords.put( "return", defaultCol);
        keywords.put( "static", defaultCol);
        keywords.put( "super", defaultCol);
        keywords.put( "switch", defaultCol);
        keywords.put( "synchronized", defaultCol);
        keywords.put( "this", defaultCol);
        keywords.put( "throw", defaultCol);
        keywords.put( "throws", defaultCol);
        keywords.put( "transient", defaultCol);
        keywords.put( "try", defaultCol);
        keywords.put( "volatile", defaultCol);
        keywords.put( "while", defaultCol);
        
        style = new SimpleAttributeSet();
    }

    public void insertString(int offset, String str, AttributeSet attr) throws BadLocationException
    {
        super.insertString(offset, str, attr);
        processChangedLines(offset, str.length());
    }

    public void remove(int offset, int length) throws BadLocationException
    {
        super.remove(offset, length);
        processChangedLines(offset, length);
    }

    public void processChangedLines(int offset, int length) throws BadLocationException
    {
        String text = getText(0, getLength());
        highlightString(Color.black, 0, getLength(), true, false);

        Set<String> keyw = keywords.keySet();
        
        String[] textParts;

        for (String keyword : keyw) {
        	
        	textParts = text.split("\"");
        	text = "";
        	
        	for (int i = 0; i < textParts.length; i++){
        		if (i%2 == 0)
        			text = text + textParts[i];
        		else
        			text = text + textParts[i].replaceAll(".", " ") + "  ";
        	}
        	
        	
        	
        	
        	Color col = keywords.get(keyword);

            Pattern p = Pattern.compile("\\b" + keyword + "\\b");
            Matcher m = p.matcher(text);

            while(m.find()) {
                    highlightString(col, m.start(), keyword.length(), true, true);
            }
        }
        
        Matcher slc = singleLineCommentDelimter.matcher(text);
        
        String left, mid, right;
        
        while(slc.find()) {
        	
            int line = rootElement.getElementIndex(slc.start());
            int endOffset = rootElement.getElement(line).getEndOffset() - 1;

            highlightString(commentColor, slc.start(), endOffset-slc.start(), true, true);
            
            left = text.substring(0, slc.start());
            mid = text.substring(slc.start(), endOffset);
            mid = mid.replaceAll(".", "X");
            right = text.substring(endOffset);
            text = left + mid + right;

        }
        

        Matcher mlcStart = multiLineCommentDelimiterStart.matcher(text);
        Matcher mlcEnd = multiLineCommentDelimiterEnd.matcher(text);

        while(mlcStart.find()) {
            if(mlcEnd.find( mlcStart.end() ))
                highlightString(multiLineCommentColor, mlcStart.start(), (mlcEnd.end()-mlcStart.start()), true, true);
            else
                highlightString(multiLineCommentColor, mlcStart.start(), getLength(), true, true);
        }


    }
    
    public void highlightString(Color col, int begin, int length, boolean flag, boolean bold)
    {
        StyleConstants.setForeground(style, col);
        StyleConstants.setBold(style, bold);
        
        setCharacterAttributes(begin, length, style, flag);
    }

}

class SyntaxHighlightingKit extends StyledEditorKit {
	
        /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public Document createDefaultDocument()
    {
        return new SyntaxHighlightingDocument();
    }
}
