package gui.filters;

import java.io.File;

import gui.GuiMain;
import gui.MenuBar;
import gui.PackagePanel;
import gui.StatementPanel;
import gui.EditorPanel;
import gui.common.FileInfo;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import opengl.CodeRenderer;
import opengl.VisCanvas;

public class CodeFileChooser extends JFileChooser{
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -3264789354189142522L;
	
	private static CodeFileChooser chooser = null;
	private TextFilter txtFilter;
	private JavaFilter jFilter;
	private TransferFilter tfFilter;
	private JpgFilter jpgFilter;
	private PngFilter pngFilter;
	private ProjectFilter pFilter;
	
	private CodeFileChooser () {
		
		txtFilter = new TextFilter();
		jFilter = new JavaFilter();
		tfFilter = new TransferFilter();
		pFilter = new ProjectFilter();
		jpgFilter = new JpgFilter();
		pngFilter = new PngFilter();
	}
	
	public static synchronized CodeFileChooser getFileChooser() {
		
		if (chooser == null)
			chooser = new CodeFileChooser();
		
		return chooser;
	}

	
	public void readTextFile() {
		
		removeFilters();
		
		GuiMain frame = GuiMain.getMainFrame();
		
		chooser.setSelectedFile(null);
		chooser.updateUI();
		
		chooser.setFileFilter(txtFilter);
		chooser.setFileFilter(jFilter);
		
		int val = chooser.showOpenDialog(frame);
		
		if (val == JFileChooser.APPROVE_OPTION && txtFilter.accept(chooser.getSelectedFile())) {
			String fileName = chooser.getSelectedFile().getName();
			frame.setTitle(frame.getTitle() + " - " + fileName);	// add file name
			
			FileInfo fileInfo = new FileInfo(chooser.getSelectedFile().getAbsolutePath());
			CodeRenderer.setSingleFile(fileInfo);
			EditorPanel.setSingleFile(fileInfo);
			
			MenuBar.getMenu().getMenuItem(MenuBar.SAVE_ITEM).setEnabled(true);
		}
	}
	
	public void readStatementLook(){
		
		removeFilters();
		
		GuiMain frame = GuiMain.getMainFrame();
		
		chooser.setSelectedFile(null);
		chooser.updateUI();
		chooser.setFileFilter(tfFilter);
		
		int val = chooser.showOpenDialog(frame);
		
		if (val == JFileChooser.APPROVE_OPTION && tfFilter.accept(chooser.getSelectedFile())) {
			StatementPanel.openStatementLook(chooser.getSelectedFile().toString());
		}
		
	}
	
	public void writeStatementLook(){
		
		removeFilters();
		
		GuiMain frame = GuiMain.getMainFrame();
		
		chooser.setSelectedFile(null);
		chooser.updateUI();
		chooser.setFileFilter(tfFilter);
		
		int val = chooser.showSaveDialog(frame);
		
		if (val == JFileChooser.APPROVE_OPTION && txtFilter.accept(chooser.getSelectedFile())) {
			StatementPanel.saveStatementLook(chooser.getSelectedFile().toString());
		}
	}
	
	public void saveScreenShot(){
		
		removeFilters();
		
		GuiMain frame = GuiMain.getMainFrame();
		
		chooser.setSelectedFile(null);
		chooser.updateUI();
		chooser.setFileFilter(jpgFilter);
		chooser.setFileFilter(pngFilter);
		chooser.setAcceptAllFileFilterUsed(false);
		
		int val = chooser.showSaveDialog(frame);		

        if (val == JFileChooser.APPROVE_OPTION) {
            File file = chooser.getSelectedFile();

            SaveFilter saveFilter = (SaveFilter)chooser.getFileFilter();
            
            if (saveFilter instanceof PngFilter)
            	file = pngFilter.addExtension(file);
            else
            	file = jpgFilter.addExtension(file);
            
            System.out.println(file.getAbsolutePath());
            
            if (file.exists()) {
            	String fileExists = "The file " + file.getName() + " exists. Do you want to overwrite it?";
            	int selection = JOptionPane.showConfirmDialog(this, fileExists, "Replace File", JOptionPane.YES_NO_OPTION);
            	
            	if (selection == JOptionPane.NO_OPTION || selection == JOptionPane.CLOSED_OPTION)
            		return;
            }
		
			VisCanvas.saveImage(file);
		}
        
        chooser.setAcceptAllFileFilterUsed(true);

		
	}
	
	public void importProject(){

		
		removeFilters();
		
		GuiMain frame = GuiMain.getMainFrame();
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		
		chooser.setSelectedFile(null);
		chooser.updateUI();
		chooser.setFileFilter(pFilter);
		
		int val = chooser.showOpenDialog(frame);
		
		if (val == JFileChooser.APPROVE_OPTION && tfFilter.accept(chooser.getSelectedFile())) 
			PackagePanel.openPackage(chooser.getSelectedFile());
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		
	}
	
	private void removeFilters(){
		
		FileFilter[] filters = chooser.getChoosableFileFilters();
		
		for (int i = 0; i < filters.length; i++){
			chooser.removeChoosableFileFilter(filters[i]);
		}
	}
}
