package opengl;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.nio.ByteBuffer;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import com.sun.opengl.util.BufferUtil;

import opengl.common.ImageFileReader;

public class FontRenderer {
	
	public static final float CH_SPACING = 0.5f, LINE_SPACING = -1.3f, CHAR_TRANS = 0.5f;;
	
	
	private static int base;  // Base Display List For The Font
    private static int[] textures = new int[2];  // Storage For Our Font Texture

    private static ByteBuffer stringBuffer = BufferUtil.newByteBuffer(256);
    private static GLU glu = new GLU();
    
    
	public static void loadGLTextures(GL gl) throws IOException {
        
        String filename = "img/Tahoma.png";
        BufferedImage img = ImageFileReader.readImage(filename);
        
        gl.glActiveTexture(GL.GL_TEXTURE0);
        gl.glGenTextures(1, textures, 0);

        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[0]);

        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR_MIPMAP_LINEAR);

        glu.gluBuild2DMipmaps(	GL.GL_TEXTURE_2D, 1, img.getWidth(), img.getHeight(), 
        					GL.GL_LUMINANCE, GL.GL_UNSIGNED_BYTE, ImageFileReader.readGrayscalePixels(img));
        
        // See errors...
		int glError = GL.GL_NO_ERROR;
    	glError = gl.glGetError();

    	if (glError != GL.GL_NO_ERROR)
    		System.out.println("Error while creating FONT Texture: " + glu.gluErrorString(glError));
    		
    }

	/**
	 * Build font display list
	 * 
	 * @param gl
	 */
	public static void buildFont(GL gl) {
        float cx;      // Holds Our X Character Coord
        float cy;      // Holds Our Y Character Coord
        float offset = 0.0625f;
        
        
        base = gl.glGenLists(256);  // Creating 256 Display Lists
        
        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[0]);  // Select Our Font Texture
        for (int loop = 0; loop < 256; loop++)      // Loop Through All 256 Lists
        {
            cx = (float) (loop % 16) / 16.0f;  // X Position Of Current Character
            cy = (float) (loop / 16) / 16.0f;  // Y Position Of Current Character

            gl.glNewList(base + loop, GL.GL_COMPILE);  // Start Building A List
            gl.glBegin(GL.GL_QUADS);      // Use A Quad For Each Character
            gl.glMultiTexCoord2f(GL.GL_TEXTURE0, cx, 1 - cy - offset);  // Texture Coord (Bottom Left)
            gl.glVertex3f(-0.5f, -0.5f, 0.0f);      // Vertex Coord (Bottom Left)
            gl.glMultiTexCoord2f(GL.GL_TEXTURE0, cx + offset, 1 - cy - offset);  // Texture Coord (Bottom Right)
            gl.glVertex3f(0.5f, -0.5f, 0.0f);      // Vertex Coord (Bottom Right)
            gl.glMultiTexCoord2f(GL.GL_TEXTURE0, cx + offset, 1 - cy);  // Texture Coord (Top Right)
            gl.glVertex3f(0.5f, 0.5f, 0.0f);      // Vertex Coord (Top Right)
            gl.glMultiTexCoord2f(GL.GL_TEXTURE0, cx, 1 - cy);    // Texture Coord (Top Left)
            gl.glVertex3f(-0.5f, 0.5f, 0.0f);      // Vertex Coord (Top Left)
            gl.glEnd();          // Done Building Our Quad (Character)
            gl.glTranslatef(CHAR_TRANS, 0, 0);      // Move To The Right Of The Character
            gl.glEndList();        // Done Building The Display List
        }            // Loop Until All 256 Are Built
    }

    /**
     * Print a string to the screen
     * 
     * @param gl
     * @param x	the x coordinate of the string
     * @param y the y coordinate of the string
     * @param string	a given input string
     * @param set	the font set (0 or 1)
     */
    public static void glPrint(GL gl, float x, float y, String string, int set) {
        
    	if (set > 1)
            set = 1;
        
        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[0]); // Select Our Font Texture
        gl.glDisable(GL.GL_DEPTH_TEST);       // Disables Depth Testing
//        gl.glMatrixMode(GL.GL_PROJECTION);     // Select The Projection Matrix
        gl.glPushMatrix();         // Store The Projection Matrix
//        gl.glLoadIdentity();         // Reset The Projection Matrix
//        gl.glOrtho(0, 640, 0, 480, -1, 1);     // Set Up An Ortho Screen
//        gl.glMatrixMode(GL.GL_MODELVIEW);     // Select The Modelview Matrix
//        gl.glPushMatrix();         // Store The Modelview Matrix
//        gl.glLoadIdentity();         // Reset The Modelview Matrix
        gl.glTranslatef(x, y, 0);       // Position The Text (0,0 - Bottom Left)
        gl.glListBase(base - 32 + (128 * set));     // Choose The Font Set (0 or 1)

        if (stringBuffer.capacity() < string.length()) {
            stringBuffer = BufferUtil.newByteBuffer(string.length());
        }

        stringBuffer.clear();
        stringBuffer.put(string.getBytes());
        stringBuffer.flip();
        
        // Write The Text To The Screen
        gl.glCallLists(string.length(), GL.GL_BYTE, stringBuffer);      
//        gl.glMatrixMode(GL.GL_PROJECTION);  // Select The Projection Matrix
//        gl.glPopMatrix();      // Restore The Old Projection Matrix
//        gl.glMatrixMode(GL.GL_MODELVIEW);  // Select The Modelview Matrix
        gl.glPopMatrix();      // Restore The Old Projection Matrix
//        gl.glEnable(GL.GL_DEPTH_TEST);    // Enables Depth Testing
    }

}
