package opengl;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import opengl.common.FadeEffect;
import opengl.common.Shader;

import logic.code.CodeItem;

public class TextRenderer {
	
	private static final int RED = 0, GREEN = 1, BLUE = 2, ALPHA = 3, TEXT = 20;
	
	private static float[] textColor;
	private static FadeEffect textEffect = new FadeEffect(TEXT, true);
	
	// OpenGL
	private static Shader textShader;
	private static int baseList;
	private static int numLists, numOldLists;
	private static GLU glu = new GLU();
	
	private static float globalAlpha = 1.0f;
	
	static {
		textColor = new float[4];
		textColor[RED] = 1.0f;
		textColor[GREEN] = 1.0f;
		textColor[BLUE] = 1.0f;
		textColor[ALPHA] = 1.0f;
		
		numLists = 0;
	}
	
	public static void setDisplayListSize(int size) {
		
		numOldLists = numLists;
		numLists = size;
	}
	
	public static void init(GL gl) {
		
		textShader = new Shader(gl, "shader/text-shader.vsh", "shader/text-shader.fsh");

		int program = textShader.getProgram();
		gl.glLinkProgramARB(program);
		gl.glUseProgram(program);
		
		gl.glUniform1iARB(gl.glGetUniformLocationARB(program, "img_tex"), 0);
		
		gl.glValidateProgramARB(program);
        textShader.checkLogInfo(gl, program);
        gl.glUseProgramObjectARB(0);
	}

	
	public static void drawText(GL gl, int i) {

		// don't render anything if the text is invisible
		if (textEffect.getAlpha() == 0.0f || globalAlpha == 0.0f)
			return;
		
		gl.glEnable(GL.GL_BLEND);
        gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
		
		gl.glUseProgram(textShader.getProgram());
		gl.glColor4f(textColor[RED], textColor[GREEN], textColor[BLUE], (float)textEffect.getAlpha()/255.0f*globalAlpha);
		gl.glCallList(baseList+i);
		gl.glUseProgram(0);
		
		gl.glDisable(GL.GL_BLEND);
		
		// See errors...
		int glError = GL.GL_NO_ERROR;
    	glError = gl.glGetError();
    	if (glError != GL.GL_NO_ERROR)
    		System.out.println("Error while drawing Text: " + glu.gluErrorString(glError));
		
	}
	
	public static Shader getTextShader() {
		return textShader;
	}
	
	public static void genLists(GL gl) {
		
		// delete old lists
		gl.glDeleteLists(baseList, numOldLists);
		baseList = gl.glGenLists(numLists);
	}
	
	public static void createList(GL gl, CodeItem codeItem, int i) {

		
		String line = "";
		float ly = 0.0f;
		gl.glNewList(baseList+i, GL.GL_COMPILE);
		
		codeItem.reset();
		
		while (codeItem.hasNext()) {
			line = codeItem.nextLine();
			if (line != null){
				// Convert tabulators to spaces
				line = line.replace("\t", "     ");
				drawLine(gl, line, 0.0f, ly);
			}
			else
				System.out.println("line is null!");
			ly += FontRenderer.LINE_SPACING;
		}
		
		gl.glEndList();
	}
	
	// Open GL...
	private static void drawLine(GL gl, String line, float x, float y) {
		
		if (line == null)
			return;
		
		gl.glEnable(GL.GL_TEXTURE_2D);
		FontRenderer.glPrint(gl, x, y, line, 0);
		gl.glDisable(GL.GL_TEXTURE_2D);
	}

	public static void setTextColor(Color color) {
		textColor[RED] = (float)color.getRed()/255.0f;
		textColor[GREEN] = (float)color.getGreen()/255.0f;
		textColor[BLUE] = (float)color.getBlue()/255.0f;
		textColor[ALPHA] = (float)color.getAlpha()/255.0f;
	}


	public static void animateText(boolean fadeIn) {
		textEffect = new FadeEffect(TEXT, fadeIn);
		
	}

	public static void setGlobalAlpha(int alpha) {
		globalAlpha = (float)alpha/100.0f;
	}

}
