package opengl.common;

import gui.GuiMain;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringWriter;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;
import javax.swing.JOptionPane;


import com.sun.opengl.util.BufferUtil;

public class Shader {
	
	private int vertShader, fragShader, program;
	private String vertFile, fragFile;
	private static GLU glu = new GLU();
	
	public Shader(GL gl, String vertFile, String fragFile) {
		this.vertFile = vertFile;
		this.fragFile = fragFile;
		
		initShader(gl);
		
	}
	
	
	public void initShader(GL gl) {
		
		vertShader = gl.glCreateShaderObjectARB(GL.GL_VERTEX_SHADER_ARB);
		fragShader = gl.glCreateShaderObjectARB(GL.GL_FRAGMENT_SHADER_ARB);
		program = gl.glCreateProgramObjectARB();
		
		String shaderSource = getShaderSource(vertFile);
		gl.glShaderSourceARB(vertShader, 1, new String[]{shaderSource}, (int[]) null, 0);
		compileShader(gl, vertShader, program);
		
		int glError = GL.GL_NO_ERROR;
		glError = gl.glGetError();
		
		if (glError != GL.GL_NO_ERROR)
			System.out.println("Error while compiling vertex shader: " + glu.gluErrorString(glError));
		else
			System.out.println("Shader compiled...");
		
		shaderSource = getShaderSource(fragFile);
		gl.glShaderSourceARB(fragShader, 1, new String[]{shaderSource}, (int[]) null, 0);
		compileShader(gl, fragShader, program);
		
		glError = GL.GL_NO_ERROR;
		glError = gl.glGetError();
		
		if (glError != GL.GL_NO_ERROR)
			System.out.println("Error while compiling fragment shader: " + glu.gluErrorString(glError));
		else
			System.out.println("Shader compiled...");

	}

	private void compileShader(GL gl, int shader, int programObject) {
	
		gl.glCompileShaderARB(shader);
		checkLogInfo(gl, shader);
		
		gl.glAttachObjectARB(programObject, shader);
	}
	
	public void checkLogInfo(GL gl, int shader) {
	    
		IntBuffer iVal = BufferUtil.newIntBuffer(1);
	    gl.glGetObjectParameterivARB(shader, GL.GL_OBJECT_INFO_LOG_LENGTH_ARB, iVal);
	
	    int length = iVal.get();
	
	    if (length <= 1) {
	        return;
	    }
	
	    ByteBuffer infoLog = BufferUtil.newByteBuffer(length);
	
	    iVal.flip();
	    gl.glGetInfoLogARB(shader, length, iVal, infoLog);
	
	    byte[] infoBytes = new byte[length];
	    infoLog.get(infoBytes);
	    System.out.println("GLSL Validation >> " + new String(infoBytes));
	}
	
	private static String getShaderSource(String filename) {
		
		String shaderSource = "";
		
		try {
			BufferedReader shaderReader = ImageFileReader.getResourceAsReader(filename);
			StringWriter shaderWriter = new StringWriter();
	
	        String line = shaderReader.readLine();
	        while (line != null) {
	            shaderWriter.write(line);
	            shaderWriter.write("\n");
	            line = shaderReader.readLine();
	        }
	
	        shaderSource = shaderWriter.getBuffer().toString();
		} catch (IOException ioe) {
			System.out.println("I could not read from the file: " +filename);
			JOptionPane.showMessageDialog(GuiMain.getMainFrame(), "I couldn't read from the file: " +filename,
					"Error", JOptionPane.OK_OPTION);
		}
		
		return shaderSource;
	}
	
	public int getProgram() {
		
		return program;
	}
	
	public int getVertShader() {
		return vertShader;
	}
	
	public int getFragShader() {
		return fragShader;
	}
	
	public void linkProgam(GL gl) {
		gl.glLinkProgram(program);
    	gl.glValidateProgramARB(program);
    	
    	checkLogInfo(gl, program);
	}
}
